@@ -1,24 +1,23 @@
-import binascii
 import datetime
 import json
+import logging
 import os
-import zipfile
-try:
-    from StringIO import StringIO ## for Python 2
-except ImportError:
-    from io import StringIO ## for Python 3
+from typing import Iterator, Optional
 
-from capstone import Cs, CS_ARCH_X86, CS_MODE_32, CS_MODE_64
+from capstone import CS_ARCH_X86, CS_MODE_32, CS_MODE_64, Cs
 
-from .BinaryInfo import BinaryInfo
-from .SmdaFunction import SmdaFunction
-from smda.common.CodeXref import CodeXref
 from smda.common.BlockLocator import BlockLocator
+from smda.common.CodeXref import CodeXref
 from smda.DisassemblyStatistics import DisassemblyStatistics
 
+from .BinaryInfo import BinaryInfo
+from .SmdaBasicBlock import SmdaBasicBlock
+from .SmdaFunction import SmdaFunction
 
-class SmdaReport(object):
+LOGGER = logging.getLogger(__name__)
 
+
+class SmdaReport:
     architecture = None
     base_addr = None
     binary_size = None
@@ -40,6 +39,8 @@ class SmdaReport(object):
     message = None
     oep = None
     sha256 = None
+    sha1 = None
+    md5 = None
     smda_version = None
     statistics = None
     # status can be "ok", "timeout", "error"
@@ -47,6 +48,7 @@ class SmdaReport(object):
     timestamp = None
     version = None
     xcfg = None
+    xheader = None
 
     # on first usage, initialize codexrefs objects for all functions based on inrefs/outrefs (requires knowledge about all functions)
     _has_codexrefs = False
@@ -62,7 +64,7 @@ class SmdaReport(object):
             self.bitness = disassembly.binary_info.bitness
             self.buffer = buffer
             self.code_areas = disassembly.binary_info.code_areas
-            self.code_sections = [section for section in disassembly.binary_info.getSections()]
+            self.code_sections = list(disassembly.binary_info.getSections())
             if self.code_sections is None:
                 self.code_sections = []
             self.component = disassembly.binary_info.component
@@ -77,19 +79,31 @@ class SmdaReport(object):
             self.message = "Analysis finished regularly."
             self.oep = disassembly.binary_info.getOep()
             self.sha256 = disassembly.binary_info.sha256
+            self.sha1 = disassembly.binary_info.sha1
+            self.md5 = disassembly.binary_info.md5
             self.smda_version = disassembly.smda_version
             self.statistics = DisassemblyStatistics(disassembly)
             self.status = disassembly.getAnalysisOutcome()
             if self.status == "timeout":
                 self.message = "Analysis was stopped when running into the timeout."
-            self.timestamp = datetime.datetime.utcnow()
+            self.timestamp = datetime.datetime.now(datetime.timezone.utc)
             self.version = disassembly.binary_info.version
             self.xcfg = self._convertCfg(disassembly, config=config)
+            self.xheader = disassembly.binary_info.getHeaderBytes()
+            self.xmetadata = {
+                "exported_functions": disassembly.binary_info.getExportedFunctions(),
+                "imported_functions": disassembly.binary_info.getImportedFunctions(),
+                "symbols": disassembly.binary_info.getSymbols(),
+            }
 
     def _convertCfg(self, disassembly, config=None):
         function_results = {}
         for function_offset in disassembly.functions:
-            if self.confidence_threshold and function_offset in disassembly.candidates and disassembly.candidates[function_offset].getConfidence() < self.confidence_threshold:
+            if (
+                self.confidence_threshold
+                and function_offset in disassembly.candidates
+                and disassembly.candidates[function_offset].getConfidence() < self.confidence_threshold
+            ):
                 continue
             smda_function = SmdaFunction(disassembly, function_offset, config=config, smda_report=self)
             function_results[function_offset] = smda_function
@@ -114,13 +128,13 @@ class SmdaReport(object):
             sum_instructions += function.num_instructions
         return sum_instructions
 
-    def getBuffer(self):
+    def getBuffer(self) -> bytes:
         return self.buffer
 
-    def getFunction(self, function_addr):
-        return self.xcfg[function_addr] if function_addr in self.xcfg else None
+    def getFunction(self, function_addr) -> Optional["SmdaFunction"]:
+        return self.xcfg.get(function_addr, None)
 
-    def getFunctions(self):
+    def getFunctions(self) -> Iterator["SmdaFunction"]:
         for _, smda_function in sorted(self.xcfg.items()):
             yield smda_function
 
@@ -129,13 +143,13 @@ class SmdaReport(object):
             if smda_function.isExported():
                 yield smda_function
 
-    def findFunctionByContainedAddress(self, inner_address):
+    def findFunctionByContainedAddress(self, inner_address) -> Optional["SmdaFunction"]:
         block = self.findBlockByContainedAddress(inner_address)
         if block is None:
             return None
         return block.smda_function
 
-    def findBlockByContainedAddress(self, inner_address):
+    def findBlockByContainedAddress(self, inner_address) -> Optional["SmdaBasicBlock"]:
         # init the block locator if it hasn't been used yet.
         if self.block_locator is None:
             self.block_locator = BlockLocator(self.getFunctions())
@@ -199,19 +213,25 @@ class SmdaReport(object):
     def fromFile(cls, file_path):
         smda_json = {}
         if os.path.isfile(file_path):
-            with open(file_path, "r") as fjson:
+            with open(file_path) as fjson:
                 smda_json = json.load(fjson)
+        else:
+            raise FileNotFoundError
         return SmdaReport.fromDict(smda_json)
 
     @classmethod
-    def fromDict(cls, report_dict):
+    def fromDict(cls, report_dict) -> Optional["SmdaReport"]:
         smda_report = cls(None)
         smda_report.architecture = report_dict["architecture"]
         smda_report.base_addr = report_dict["base_addr"]
         smda_report.binary_size = report_dict["binary_size"]
         smda_report.bitness = report_dict["bitness"]
         smda_report.code_areas = report_dict["code_areas"]
-        smda_report.code_sections = [("", section[1], section[2]) for section in report_dict["code_sections"]] if "code_sections" in report_dict else []
+        smda_report.code_sections = (
+            [("", section[1], section[2]) for section in report_dict["code_sections"]]
+            if "code_sections" in report_dict
+            else []
+        )
         smda_report.confidence_threshold = report_dict["confidence_threshold"]
         smda_report.disassembly_errors = report_dict["disassembly_errors"]
         smda_report.execution_time = report_dict["execution_time"]
@@ -229,10 +249,12 @@ class SmdaReport(object):
                 smda_report.is_library = report_dict["metadata"]["is_library"]
             if "version" in report_dict["metadata"]:
                 smda_report.version = report_dict["metadata"]["version"]
-            smda_report.is_buffer = report_dict["metadata"]["is_buffer"] if "is_buffer" in report_dict["metadata"] else False
+            smda_report.is_buffer = report_dict["metadata"].get("is_buffer", False)
         smda_report.message = report_dict["message"]
-        smda_report.oep = report_dict["oep"] if "oep" in report_dict else None
+        smda_report.oep = report_dict.get("oep", None)
         smda_report.sha256 = report_dict["sha256"]
+        smda_report.sha1 = report_dict.get("sha1", None)
+        smda_report.md5 = report_dict.get("md5", None)
         smda_report.smda_version = report_dict["smda_version"]
         smda_report.statistics = DisassemblyStatistics.fromDict(report_dict["statistics"])
         smda_report.status = report_dict["status"]
@@ -242,22 +264,39 @@ class SmdaReport(object):
         binary_info.base_addr = smda_report.base_addr
         binary_info.binary_size = smda_report.binary_size
         binary_info.oep = smda_report.oep
-        smda_report.xcfg = {int(function_addr): SmdaFunction.fromDict(function_dict, binary_info=binary_info, version=smda_report.smda_version, smda_report=smda_report) for function_addr, function_dict in report_dict["xcfg"].items()}
+        smda_report.xcfg = {
+            int(function_addr): SmdaFunction.fromDict(
+                function_dict,
+                binary_info=binary_info,
+                version=smda_report.smda_version,
+                smda_report=smda_report,
+            )
+            for function_addr, function_dict in report_dict["xcfg"].items()
+        }
+        smda_report.xheader = bytes.fromhex(report_dict["xheader"]) if "xheader" in report_dict else None
+        smda_report.xmetadata = report_dict.get("xmetadata", None)
         return smda_report
 
-    def toDict(self):
+    def toDict(self) -> dict:
+        transformed_code_sections = []
+        if self.code_sections:
+            for section in self.code_sections:
+                if section is not None:
+                    transformed_code_sections.append(("", section[1], section[2]))
+                else:
+                    transformed_code_sections.append(("", 0, 0))
         return {
             "architecture": self.architecture,
             "base_addr": self.base_addr,
             "binary_size": self.binary_size,
             "bitness": self.bitness,
             "code_areas": self.code_areas,
-            "code_sections": [("", section[1], section[2]) for section in self.code_sections],
+            "code_sections": transformed_code_sections,
             "confidence_threshold": self.confidence_threshold,
             "disassembly_errors": self.disassembly_errors,
             "execution_time": self.execution_time,
             "identified_alignment": self.identified_alignment,
-            "metadata" : {
+            "metadata": {
                 "binweight": self.binweight,
                 "component": self.component,
                 "family": self.family,
@@ -269,14 +308,23 @@ class SmdaReport(object):
             "message": self.message,
             "oep": self.oep,
             "sha256": self.sha256,
+            "sha1": self.sha1,
+            "md5": self.md5,
             "smda_version": self.smda_version,
-            "statistics": self.statistics.toDict(),
+            "statistics": self.statistics.toDict() if self.statistics else {},
             "status": self.status,
             "timestamp": self.timestamp.strftime("%Y-%m-%dT%H-%M-%S"),
-            "xcfg": {function_addr: smda_function.toDict() for function_addr, smda_function in self.xcfg.items()}
+            "xcfg": {function_addr: smda_function.toDict() for function_addr, smda_function in self.xcfg.items()},
+            "xheader": self.xheader.hex() if self.xheader else "",
+            "xmetadata": self.xmetadata,
         }
 
+    def toFile(self, output_filepath) -> None:
+        with open(output_filepath, "w") as fout:
+            json.dump(self.toDict(), fout, indent=1, sort_keys=True)
+            LOGGER.info(f"SmdaReport saved to: {output_filepath}")
+
     def __str__(self):
         if self.status == "error":
-            return "{:>6.3f}s -> {}".format(self.execution_time, self.message)
-        return "{:>6.3f}s -> (architecture: {}.{}bit, base_addr: 0x{:08x}): {} functions".format(self.execution_time, self.architecture, self.bitness, self.base_addr, len(self.xcfg))
+            return f"{self.execution_time:>6.3f}s -> {self.message}"
+        return f"{self.execution_time:>6.3f}s -> (architecture: {self.architecture}.{self.bitness}bit, base_addr: 0x{self.base_addr:08x}): {len(self.xcfg)} functions"
