@@ -1,5 +1,4 @@
-class BackendInterface(object):
-
+class BackendInterface:
     def __init__(self):
         pass
 
@@ -24,7 +23,7 @@ class BackendInterface(object):
     def getInstructionBytes(self, offset):
         raise NotImplementedError
 
-    def getFunctionSymbols(self):
+    def getFunctionSymbols(self, demangle=False):
         raise NotImplementedError
 
     def getBaseAddr(self):
