@@ -1,6 +1,7 @@
 import heapq
 
-class PriorityQueue(object):
+
+class PriorityQueue:
     def __init__(self, content=None):
         if content is None:
             content = []
@@ -27,10 +28,11 @@ class PriorityQueue(object):
 
     def add(self, element):
         self.heap.append(element)
-        heapq._siftdown_max(self.heap, 0, len(self.heap)-1)
+        heapq._siftdown_max(self.heap, 0, len(self.heap) - 1)
 
-    def update(self):
-        heapq._heapify_max(self.heap)
+    def update(self, target_candidate=None):
+        if target_candidate is None:
+            heapq._heapify_max(self.heap)
 
     def __str__(self):
         return str(self.heap)
